@@ -1,4 +1,4 @@
-/* gltext, Copyright (c) 2001, 2002, 2003, 2004 Jamie Zawinski <jwz@jwz.org>
+/* gltext, Copyright (c) 2001-2008 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -9,30 +9,12 @@
  * implied warranty.
  */
 
-#include <X11/Intrinsic.h>
-
-extern XtAppContext app;
-
-#define PROGCLASS	"GLText"
-#define HACK_INIT	init_text
-#define HACK_DRAW	draw_text
-#define HACK_RESHAPE	reshape_text
-#define HACK_HANDLE_EVENT text_handle_event
-#define EVENT_MASK	PointerMotionMask
-#define sws_opts	xlockmore_opts
-
-#define DEF_TEXT        "(default)"
-#define DEF_SPIN        "XYZ"
-#define DEF_WANDER      "True"
-
-#define DEFAULTS	"*delay:	20000       \n" \
-			"*showFPS:      False       \n" \
-			"*wireframe:    False       \n" \
-			"*spin:       " DEF_SPIN   "\n" \
-			"*wander:     " DEF_WANDER "\n" \
-			"*text:       " DEF_TEXT   "\n"
-
+#define DEFAULTS	"*delay:	20000        \n" \
+			"*showFPS:      False        \n" \
+			"*wireframe:    False        \n" \
 
+# define refresh_text 0
+# define release_text 0
 #define SMOOTH_TUBE       /* whether to have smooth or faceted tubes */
 
 #ifdef SMOOTH_TUBE
@@ -50,18 +32,21 @@ extern XtAppContext app;
 #include "tube.h"
 #include "rotator.h"
 #include "gltrackball.h"
-#include <time.h>
-#include <sys/time.h>
+
 #include <ctype.h>
 
 #ifdef USE_GL /* whole file */
 
+#define DEF_TEXT        "(default)"
+#define DEF_PROGRAM     "(default)"
+#define DEF_SPIN        "XYZ"
+#define DEF_WANDER      "True"
+#define DEF_FACE_FRONT  "True"
+
 #ifdef HAVE_UNAME
 # include <sys/utsname.h>
 #endif /* HAVE_UNAME */
 
-
-#include <GL/glu.h>
 #include "glutstroke.h"
 #include "glut_roman.h"
 #define GLUT_FONT (&glutStrokeRoman)
@@ -69,9 +54,10 @@ extern XtAppContext app;
 
 typedef struct {
   GLXContext *glx_context;
-  rotator *rot;
+  rotator *rot, *rot2;
   trackball_state *trackball;
   Bool button_down_p;
+  Bool spinx, spiny, spinz;
 
   GLuint text_list;
 
@@ -80,35 +66,45 @@ typedef struct {
   int ccolor;
 
   char *text;
+  int reload;
+
+  time_t last_update;
 
 } text_configuration;
 
 static text_configuration *tps = NULL;
 
 static char *text_fmt;
+static char *program_str;
 static char *do_spin;
 static Bool do_wander;
+static Bool face_front_p;
 
 static XrmOptionDescRec opts[] = {
-  { "-text",   ".text",   XrmoptionSepArg, 0 },
-  { "-spin",   ".spin",   XrmoptionSepArg, 0 },
-  { "+spin",   ".spin",   XrmoptionNoArg, "" },
-  { "-wander", ".wander", XrmoptionNoArg, "True" },
-  { "+wander", ".wander", XrmoptionNoArg, "False" }
+  { "-text",    ".text",      XrmoptionSepArg, 0 },
+  { "-program", ".program",   XrmoptionSepArg, 0 },
+  { "-spin",    ".spin",      XrmoptionSepArg, 0 },
+  { "+spin",    ".spin",      XrmoptionNoArg, "" },
+  { "-wander",  ".wander",    XrmoptionNoArg, "True" },
+  { "+wander",  ".wander",    XrmoptionNoArg, "False" },
+  { "-front",   ".faceFront", XrmoptionNoArg, "True" },
+  { "+front",   ".faceFront", XrmoptionNoArg, "False" }
 };
 
 static argtype vars[] = {
-  {&text_fmt,  "text",   "Text",   DEF_TEXT,   t_String},
-  {&do_spin,   "spin",   "Spin",   DEF_SPIN,   t_String},
-  {&do_wander, "wander", "Wander", DEF_WANDER, t_Bool},
+  {&text_fmt,     "text",      "Text",      DEF_TEXT,       t_String},
+  {&program_str,  "program",   "Program",   DEF_PROGRAM,    t_String},
+  {&do_spin,      "spin",      "Spin",      DEF_SPIN,       t_String},
+  {&do_wander,    "wander",    "Wander",    DEF_WANDER,     t_Bool},
+  {&face_front_p, "faceFront", "FaceFront", DEF_FACE_FRONT, t_Bool},
 };
 
-ModeSpecOpt sws_opts = {countof(opts), opts, countof(vars), vars, NULL};
+ENTRYPOINT ModeSpecOpt text_opts = {countof(opts), opts, countof(vars), vars, NULL};
 
 
 /* Window management, etc
  */
-void
+ENTRYPOINT void
 reshape_text (ModeInfo *mi, int width, int height)
 {
   GLfloat h = (GLfloat) height / (GLfloat) width;
@@ -135,7 +131,7 @@ gl_init (ModeInfo *mi)
   text_configuration *tp = &tps[MI_SCREEN(mi)];
   int wire = MI_IS_WIREFRAME(mi);
 
-  static GLfloat pos[4] = {5.0, 5.0, 10.0, 1.0};
+  static const GLfloat pos[4] = {5.0, 5.0, 10.0, 1.0};
 
   if (!wire)
     {
@@ -185,7 +181,42 @@ parse_text (ModeInfo *mi)
 
   if (tp->text) free (tp->text);
 
-  if (!text_fmt || !*text_fmt || !strcmp(text_fmt, "(default)"))
+  if (program_str && *program_str && !!strcmp(program_str, "(default)"))
+    {
+      FILE *p;
+      int i;
+      char buf[1024];
+      sprintf (buf, "( %.900s ) 2>&1", program_str);
+      p = popen (buf, "r");
+      if (! p)
+        sprintf (buf, "error running '%.900s'", program_str);
+      else
+        {
+          char *out = buf;
+          char *end = out + sizeof(buf) - 1;
+          int n;
+          do {
+            n = fread (out, 1, end - out, p);
+            if (n > 0)
+              out += n;
+            *out = 0;
+          } while (n > 0);
+          fclose (p);
+        }
+
+      /* Truncate it to 10 lines */
+      {
+        char *s = buf;
+        for (i = 0; i < 10; i++)
+          if (s && (s = strchr (s, '\n')))
+            s++;
+        if (s) *s = 0;
+      }
+
+      tp->text = strdup (buf);
+      tp->reload = 5;
+    }
+  else if (!text_fmt || !*text_fmt || !strcmp(text_fmt, "(default)"))
     {
 # ifdef HAVE_UNAME
       struct utsname uts;
@@ -207,8 +238,34 @@ parse_text (ModeInfo *mi)
           sprintf(tp->text, "%s\n%s %s.%s",
                   uts.nodename, uts.sysname, uts.version, uts.release);
 #  elif defined(__APPLE__)  /* MacOS X + XDarwin */
-          sprintf(tp->text, "%s\n%s %s\n%s",
-                  uts.nodename, uts.sysname, uts.release, uts.machine);
+          {
+            const char *file = 
+              "/System/Library/CoreServices/SystemVersion.plist";
+            FILE *f = fopen (file, "r");
+            char *pbv = 0, *pn = 0, *puvv = 0;
+            if (f) {
+              char *s, buf[255];
+
+              while (fgets (buf, sizeof(buf)-1, f)) {
+#               define GRAB(S,V)					\
+                if (strstr(buf, S)) {					\
+                  fgets (buf, sizeof(buf)-1, f);			\
+                  if ((s = strchr (buf, '>'))) V = strdup(s+1); 	\
+                  if ((s = strchr (V, '<'))) *s = 0;		 	\
+                }
+                GRAB ("ProductName", pn)
+                GRAB ("ProductBuildVersion", pbv)
+                GRAB ("ProductUserVisibleVersion", puvv)
+#               undef GRAB
+              }
+            }
+            if (pbv)
+              sprintf (tp->text, "%s\n%s\n%s\n%s", 
+                       uts.nodename, pn, puvv, uts.machine);
+            else
+              sprintf(tp->text, "%s\n%s %s\n%s",
+                      uts.nodename, uts.sysname, uts.release, uts.machine);
+          }
 #  else
           sprintf(tp->text, "%s\n%s %s",
                   uts.nodename, uts.sysname, uts.release);
@@ -236,19 +293,20 @@ parse_text (ModeInfo *mi)
       strftime (tp->text, L-1, text_fmt, tm);
       if (!*tp->text)
         sprintf (tp->text, "strftime error:\n%s", text_fmt);
+      tp->reload = 1;
     }
 
   latin1_to_ascii (tp->text);
 }
 
 
-Bool
+ENTRYPOINT Bool
 text_handle_event (ModeInfo *mi, XEvent *event)
 {
   text_configuration *tp = &tps[MI_SCREEN(mi)];
 
   if (event->xany.type == ButtonPress &&
-      event->xbutton.button & Button1)
+      event->xbutton.button == Button1)
     {
       tp->button_down_p = True;
       gltrackball_start (tp->trackball,
@@ -257,11 +315,21 @@ text_handle_event (ModeInfo *mi, XEvent *event)
       return True;
     }
   else if (event->xany.type == ButtonRelease &&
-           event->xbutton.button & Button1)
+           event->xbutton.button == Button1)
     {
       tp->button_down_p = False;
       return True;
     }
+  else if (event->xany.type == ButtonPress &&
+           (event->xbutton.button == Button4 ||
+            event->xbutton.button == Button5 ||
+            event->xbutton.button == Button6 ||
+            event->xbutton.button == Button7))
+    {
+      gltrackball_mousewheel (tp->trackball, event->xbutton.button, 10,
+                              !!event->xbutton.state);
+      return True;
+    }
   else if (event->xany.type == MotionNotify &&
            tp->button_down_p)
     {
@@ -275,7 +343,7 @@ text_handle_event (ModeInfo *mi, XEvent *event)
 }
 
 
-void 
+ENTRYPOINT void 
 init_text (ModeInfo *mi)
 {
   text_configuration *tp;
@@ -300,17 +368,18 @@ init_text (ModeInfo *mi)
   }
 
   {
-    Bool spinx=False, spiny=False, spinz=False;
     double spin_speed   = 0.5;
     double wander_speed = 0.02;
+    double tilt_speed   = 0.03;
     double spin_accel   = 0.5;
 
     char *s = do_spin;
     while (*s)
       {
-        if      (*s == 'x' || *s == 'X') spinx = True;
-        else if (*s == 'y' || *s == 'Y') spiny = True;
-        else if (*s == 'z' || *s == 'Z') spinz = True;
+        if      (*s == 'x' || *s == 'X') tp->spinx = True;
+        else if (*s == 'y' || *s == 'Y') tp->spiny = True;
+        else if (*s == 'z' || *s == 'Z') tp->spinz = True;
+        else if (*s == '0') ;
         else
           {
             fprintf (stderr,
@@ -321,12 +390,15 @@ init_text (ModeInfo *mi)
         s++;
       }
 
-    tp->rot = make_rotator (spinx ? spin_speed : 0,
-                            spiny ? spin_speed : 0,
-                            spinz ? spin_speed : 0,
+    tp->rot = make_rotator (tp->spinx ? spin_speed : 0,
+                            tp->spiny ? spin_speed : 0,
+                            tp->spinz ? spin_speed : 0,
                             spin_accel,
                             do_wander ? wander_speed : 0,
                             False);
+    tp->rot2 = (face_front_p
+                ? make_rotator (0, 0, 0, 0, tilt_speed, True)
+                : 0);
     tp->trackball = gltrackball_init ();
   }
 
@@ -350,7 +422,7 @@ init_text (ModeInfo *mi)
 
 
 static int
-fill_character (GLUTstrokeFont font, int c, Bool wire)
+fill_character (GLUTstrokeFont font, int c, Bool wire, int *polysP)
 {
   GLfloat tube_width = 10;
 
@@ -379,16 +451,16 @@ fill_character (GLUTstrokeFont font, int c, Bool wire)
             int smooth = False;
 # endif
             if (j != stroke->num_coords)
-              tube (lx,       ly,       0,
-                    coord->x, coord->y, 0,
-                    tube_width,
-                    tube_width * 0.15,
-                    TUBE_FACES, smooth, True, wire);
+              *polysP += tube (lx,       ly,       0,
+                               coord->x, coord->y, 0,
+                               tube_width,
+                               tube_width * 0.15,
+                               TUBE_FACES, smooth, True, wire);
             lx = coord->x;
             ly = coord->y;
           }
       }
-      return (int) (ch->right + tube_width/2);
+      return (int) (ch->right + tube_width);
     }
   return 0;
 }
@@ -428,9 +500,10 @@ text_extents (const char *string, int *wP, int *hP)
 }
 
 
-static void
+static unsigned long
 fill_string (const char *string, Bool wire)
 {
+  int polys = 0;
   const char *s, *start;
   int line_height = GLUT_FONT->top - GLUT_FONT->bottom;
   int off;
@@ -467,7 +540,7 @@ fill_string (const char *string, Bool wire)
           {
             glPushMatrix();
             glTranslatef(x, y, 0);
-            off = fill_character (GLUT_FONT, *s2, wire);
+            off = fill_character (GLUT_FONT, *s2, wire, &polys);
             x += off;
             glPopMatrix();
           }
@@ -480,10 +553,11 @@ fill_string (const char *string, Bool wire)
       }
     else
       s++;
+  return polys;
 }
 
 
-void
+ENTRYPOINT void
 draw_text (ModeInfo *mi)
 {
   text_configuration *tp = &tps[MI_SCREEN(mi)];
@@ -491,19 +565,21 @@ draw_text (ModeInfo *mi)
   Window window = MI_WINDOW(mi);
   int wire = MI_IS_WIREFRAME(mi);
 
-  static GLfloat color[4] = {0.0, 0.0, 0.0, 1.0};
-  static GLfloat white[4] = {1.0, 1.0, 1.0, 1.0};
+  GLfloat white[4] = {1.0, 1.0, 1.0, 1.0};
+  GLfloat color[4] = {0.0, 0.0, 0.0, 1.0};
 
   if (!tp->glx_context)
     return;
 
-  if (strchr (text_fmt, '%'))
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(tp->glx_context));
+
+  if (tp->reload)
     {
-      static time_t last_update = -1;
-      time_t now = time ((time_t *) 0);
-      if (now != last_update) /* do it once a second */
-        parse_text (mi);
-      last_update = now;
+      if (time ((time_t *) 0) >= tp->last_update + tp->reload)
+        {
+          parse_text (mi);
+          tp->last_update = time ((time_t *) 0);
+        }
     }
 
   glShadeModel(GL_SMOOTH);
@@ -512,6 +588,7 @@ draw_text (ModeInfo *mi)
   glEnable(GL_NORMALIZE);
   glEnable(GL_CULL_FACE);
 
+
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
   glPushMatrix ();
@@ -527,10 +604,21 @@ draw_text (ModeInfo *mi)
 
     gltrackball_rotate (tp->trackball);
 
-    get_rotation (tp->rot, &x, &y, &z, !tp->button_down_p);
-    glRotatef (x * 360, 1.0, 0.0, 0.0);
-    glRotatef (y * 360, 0.0, 1.0, 0.0);
-    glRotatef (z * 360, 0.0, 0.0, 1.0);
+    if (face_front_p)
+      {
+        double max = 90;
+        get_position (tp->rot2, &x, &y, &z, !tp->button_down_p);
+        if (tp->spinx) glRotatef (max/2 - x*max, 1, 0, 0);
+        if (tp->spiny) glRotatef (max/2 - y*max, 0, 1, 0);
+        if (tp->spinz) glRotatef (max/2 - z*max, 0, 0, 1);
+      }
+    else
+      {
+        get_rotation (tp->rot, &x, &y, &z, !tp->button_down_p);
+        glRotatef (x * 360, 1, 0, 0);
+        glRotatef (y * 360, 0, 1, 0);
+        glRotatef (z * 360, 0, 0, 1);
+      }
   }
 
 
@@ -546,7 +634,7 @@ draw_text (ModeInfo *mi)
 
   glScalef(0.01, 0.01, 0.01);
 
-  fill_string(tp->text, wire);
+  mi->polygon_count = fill_string(tp->text, wire);
 
   glPopMatrix ();
 
@@ -556,4 +644,6 @@ draw_text (ModeInfo *mi)
   glXSwapBuffers(dpy, window);
 }
 
+XSCREENSAVER_MODULE_2 ("GLText", gltext, text)
+
 #endif /* USE_GL */
